package ru.dbotthepony.mc.otm.block.entity;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.util.LazyOptional;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.capability.AbstractStorageGridCell;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.MatteryMachineEnergyStorage;
import ru.dbotthepony.mc.otm.container.MatteryContainer;
import ru.dbotthepony.mc.otm.core.Fraction;
import ru.dbotthepony.mc.otm.menu.DriveRackMenu;
import ru.dbotthepony.mc.otm.storage.ItemStackWrapper;
import ru.dbotthepony.mc.otm.storage.VirtualComponent;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import java.math.BigDecimal;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class BlockEntityDriveRack extends BlockEntityMatteryPowered {
	public BlockEntityDriveRack(BlockPos p_155229_, BlockState p_155230_) {
		super(Registry.BlockEntities.DRIVE_RACK, p_155229_, p_155230_);
		energy = new MatteryMachineEnergyStorage(this, MatteryMachineEnergyStorage.MachineType.WORKER, new Fraction(80_000));
	}

	public final MatteryContainer drives = new MatteryContainer(this::setChanged, 4) {
		@Override
		public void setChanged(int slot, ItemStack new_state, ItemStack old_state) {
			super.setChanged(slot, new_state, old_state);

			old_state.getCapability(MatteryCapability.DRIVE).ifPresent((drive) -> {
				cell.computeIfAbsent(drive.storageIdentity(), VirtualComponent::new).remove(drive);
			});

			new_state.getCapability(MatteryCapability.DRIVE).ifPresent((drive) -> {
				cell.computeIfAbsent(drive.storageIdentity(), VirtualComponent::new).add(drive);
			});
		}
	};

	public final AbstractStorageGridCell cell = new AbstractStorageGridCell();

	@Override
	public void load(CompoundTag nbt) {
		super.load(nbt);
		drives.deserializeNBT(nbt.getCompound("drives"));
	}

	@Override
	public void saveAdditional(CompoundTag nbt) {
		super.saveAdditional(nbt);
		nbt.put("drives", drives.serializeNBT());
	}

	@Override
	public void setLevel(Level p_155231_) {
		super.setLevel(p_155231_);

		cell.scheduleDiscoverNeighbours(p_155231_, getBlockPos());
	}

	private static final TranslatableComponent NAME = new TranslatableComponent("block.overdrive_that_matters.drive_rack");

	@Override
	protected Component getDefaultDisplayName() {
		return NAME;
	}

	@Nullable
	@Override
	public AbstractContainerMenu createMenu(int containerID, Inventory inventory, Player ply) {
		return new DriveRackMenu(containerID, inventory, this);
	}

	@Nonnull
	@Override
	public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
		if (cap == MatteryCapability.STORAGE_CELL) {
			return cell.get().cast();
		}

		return super.getCapability(cap, side);
	}

	@Override
	public void invalidateCaps() {
		super.invalidateCaps();
		cell.invalidate();
	}

	@Override
	public void reviveCaps() {
		super.reviveCaps();
		cell.revive();
	}
}
