package ru.dbotthepony.mc.otm.capability;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraftforge.common.util.INBTSerializable;
import ru.dbotthepony.mc.otm.core.Fraction;

import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;
import java.math.BigDecimal;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class MatteryMachineEnergyStorage implements IMatteryEnergyStorage, INBTSerializable<CompoundTag> {
	public enum MachineType {
		WORKER,
		GENERATOR,
		CAPACITOR,
	}

	public static final Fraction DEFAULT_MAX_RECEIVE = new Fraction(200);
	public static final Fraction DEFAULT_MAX_EXTRACT = new Fraction(200);
	public static final Fraction DEFAULT_MAX_CAPACITY = new Fraction(60000);

	protected Fraction energy_stored = Fraction.ZERO;
	protected Fraction energy_stored_max;
	protected Fraction max_input;
	protected Fraction max_output;
	protected final MachineType machine_type;
	protected final BlockEntity listener;

	public MatteryMachineEnergyStorage(BlockEntity listener, MachineType type, Fraction capacity) {
		this(listener, type, capacity,  DEFAULT_MAX_RECEIVE, DEFAULT_MAX_EXTRACT);
	}

	public MatteryMachineEnergyStorage(BlockEntity listener, MachineType type) {
		this(listener, type, DEFAULT_MAX_CAPACITY);
	}

	public MatteryMachineEnergyStorage(BlockEntity listener, MachineType type, Fraction capacity, Fraction maxReceive, Fraction maxExtract) {
		this.listener = listener;
		energy_stored_max = capacity;
		max_input = maxReceive;
		max_output = maxExtract;
		machine_type = type;
	}

	@Nonnull
	@Override
	public Fraction extractEnergyOuter(Fraction howMuch, boolean simulate) {
		if (machine_type == MachineType.WORKER) {
			return Fraction.ZERO;
		}

		return extractEnergyInner(howMuch, simulate);
	}

	@Nonnull
	@Override
	public Fraction extractEnergyInner(Fraction howMuch, boolean simulate) {
		Fraction new_energy = energy_stored.minus(howMuch.min(max_output)).moreThanZero();
		Fraction diff = energy_stored.minus(new_energy);

		if (!simulate && !energy_stored.equalsCompact(new_energy)) {
			energy_stored = new_energy;
			listener.setChanged();
		}

		return diff;
	}

	@Nonnull
	@Override
	public Fraction receiveEnergyOuter(Fraction howMuch, boolean simulate) {
		if (machine_type == MachineType.GENERATOR) {
			return Fraction.ZERO;
		}

		return receiveEnergyInner(howMuch, simulate);
	}

	@Nonnull
	@Override
	public Fraction receiveEnergyInner(Fraction howMuch, boolean simulate) {
		Fraction new_energy = energy_stored.plus(howMuch.min(max_input)).min(energy_stored_max);
		Fraction diff = new_energy.minus(energy_stored);

		if (!simulate && !energy_stored.equalsCompact(new_energy)) {
			energy_stored = new_energy;
			listener.setChanged();
		}

		return diff;
	}

	@Nonnull
	@Override
	public Fraction getBatteryLevel() {
		return energy_stored;
	}

	@Nonnull
	@Override
	public Fraction getMaxBatteryLevel() {
		return energy_stored_max;
	}

	@Override
	public boolean canExtract() {
		return machine_type != MachineType.WORKER;
	}

	@Override
	public boolean canReceive() {
		return machine_type != MachineType.GENERATOR;
	}

	@Override
	@Nonnull
	public CompoundTag serializeNBT() {
		CompoundTag tag = new CompoundTag();
		tag.put("energy_stored", energy_stored.serializeNBT());
		tag.put("energy_stored_max", energy_stored_max.serializeNBT());
		tag.put("max_input", max_input.serializeNBT());
		tag.put("max_output", max_output.serializeNBT());
		return tag;
	}

	@Override
	public void deserializeNBT(CompoundTag nbt) {
		if (nbt.contains("energy_stored"))
			energy_stored = Fraction.deserializeNBT(nbt.get("energy_stored"));

		if (nbt.contains("energy_stored_max"))
			energy_stored_max = Fraction.deserializeNBT(nbt.get("energy_stored_max"));

		if (nbt.contains("max_input"))
			max_input = Fraction.deserializeNBT(nbt.get("max_input"));

		if (nbt.contains("max_output"))
			max_output = Fraction.deserializeNBT(nbt.get("max_output"));
	}
}
