package ru.dbotthepony.mc.otm.client.screen;

import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.ChatFormatting;
import net.minecraft.client.Minecraft;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.entity.player.Inventory;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.android.AndroidResearch;
import ru.dbotthepony.mc.otm.android.AndroidResearchType;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.android.AndroidCapabilityPlayer;
import ru.dbotthepony.mc.otm.client.render.RGBAColor;
import ru.dbotthepony.mc.otm.client.render.RenderHelper;
import ru.dbotthepony.mc.otm.client.screen.widget.PowerGaugePanel;
import ru.dbotthepony.mc.otm.menu.AndroidStationMenu;
import ru.dbotthepony.mc.otm.menu.slot.MatterySlot;
import ru.dbotthepony.mc.otm.client.screen.panels.*;

import javax.annotation.Nullable;
import java.util.*;

public class AndroidStationScreen extends MatteryScreen<AndroidStationMenu> {
	class AndroidResearchButton extends EditablePanel {
		public static final int BUTTON_SIZE = 18;
		private final AndroidResearch node;

		public static final RGBAColor RESEARCHED = new RGBAColor(150, 150, 200);
		public static final RGBAColor CAN_BE_RESEARCHED = new RGBAColor(150, 200, 150);
		public static final RGBAColor CAN_NOT_BE_RESEARCHED = new RGBAColor(200, 150, 150);

		public AndroidResearchButton(@Nullable EditablePanel parent, AndroidResearch node) {
			super(AndroidStationScreen.this, parent, 0, 0, BUTTON_SIZE, BUTTON_SIZE);
			this.node = node;
			setDockMargin(2, 2, 2, 2);
		}

		@Override
		protected void innerRender(PoseStack stack, float mouse_x, float mouse_y, float flag) {
			Minecraft.getInstance().player.getCapability(MatteryCapability.ANDROID).ifPresent(_cap -> {
				if (_cap instanceof AndroidCapabilityPlayer cap) {
					if (node.isResearched()) {
						RESEARCHED.setSystemColor();
					} else if (node.canResearch()) {
						CAN_BE_RESEARCHED.setSystemColor();
					} else {
						CAN_NOT_BE_RESEARCHED.setSystemColor();
					}

					var icon = node.getIcon();

					if (icon != null) {
						icon.render(stack, 0, 0, getWidth(), getHeight());
					} else {
						RenderHelper.drawRect(stack, 0, 0, getWidth(), getHeight());
					}

					var text = node.getIconText();

					if (text != null) {
						font.drawShadow(stack, text, getWidth() - font.width(text), getHeight() - font.lineHeight, 0xFFFFFFFF);
					}
				}
			});
		}

		@Override
		protected boolean mouseClickedInner(double mouse_x, double mouse_y, int flag) {
			node.researchClient();
			return true;
		}

		@Override
		protected boolean innerRenderTooltips(PoseStack stack, float mouse_x, float mouse_y, float flag) {
			if (isHovered()) {
				var list = new ArrayList<>(node.getTooltip());

				if (node.isResearched()) {
					list.add(new TranslatableComponent("otm.android_station.research.researched").withStyle(ChatFormatting.DARK_AQUA));
				} else if (node.canResearch()) {
					list.add(new TranslatableComponent("otm.android_station.research.can_be_researched").withStyle(ChatFormatting.DARK_GREEN));
				} else {
					list.add(new TranslatableComponent("otm.android_station.research.can_not_be_researched").withStyle(ChatFormatting.DARK_RED));
				}

				renderComponentTooltip(stack, list, (int) mouse_x, (int) mouse_y);
			}

			return isHovered();
		}
	}

	public static final int FRAME_WIDTH = 210;
	public static final int GRID_WIDTH = 5;
	public static final int FRAME_HEIGHT = 120;

	public AndroidStationScreen(AndroidStationMenu p_97741_, Inventory p_97742_, Component p_97743_) {
		super(p_97741_, p_97742_, p_97743_);
	}

	private final EditablePanel[] rows = new EditablePanel[100];
	private final Set<AndroidResearchType<?>> seen = new HashSet<>();
	private float next_x = 0;
	private final float[] rows_width = new float[100];
	private final AndroidResearchButton[][] created_buttons = new AndroidResearchButton[100][1000];
	private final int[] created_buttons_idx = new int[100];

	private void dive(AndroidCapabilityPlayer cap, AndroidResearchType<?> research, int level) {
		if (seen.contains(research))
			return;

		seen.add(research);

		if (rows[level] == null) {
			rows[level] = new EditablePanel(this, canvas, 0, level * 22, 10000, 22) {
				@Override
				protected boolean mouseClickedInner(double mouse_x, double mouse_y, int flag) {
					return false;
				}

				@Override
				protected boolean mouseReleasedInner(double mouse_x, double mouse_y, int flag) {
					return false;
				}

				@Override
				protected boolean mouseDraggedInner(double mouse_x, double mouse_y, int flag, double drag_x, double drag_y) {
					return false;
				}
			};
		}

		var row = rows[level];
		var button = new AndroidResearchButton(row, cap.getResearch(research));
		button.setPos(next_x + rows_width[level], 2);
		created_buttons[level][created_buttons_idx[level]] = button;
		created_buttons_idx[level]++;

		rows_width[level] += 22;

		for (var _research : research.getUnlocks()) {
			dive(cap, _research, level + 1);
		}

		if (level > 0) {
			for (var _research : research.getPrerequisites()) {
				dive(cap, _research, level - 1);
			}
		}
	}

	private DraggableCanvasPanel canvas;
	private FramePanel research;

	private void openResearchTree() {
		var window = this.minecraft.getWindow();
		research = new FramePanel(this, null, 0, 0, window.getGuiScaledWidth() * 0.8f, window.getGuiScaledHeight() * 0.8f, new TranslatableComponent("otm.gui.android_research"));

		canvas = new DraggableCanvasPanel(this, research, 0, 0, GRID_WIDTH * 22, 0) {
			@Override
			protected void innerRender(PoseStack stack, float mouse_x, float mouse_y, float flag) {
				RenderHelper.setDrawColor(RGBAColor.BLACK);
				RenderHelper.drawRect(stack, 0, 0, width, height);
			}
		};

		minecraft.player.getCapability(MatteryCapability.ANDROID).ifPresent(_cap -> {
			if (_cap instanceof AndroidCapabilityPlayer cap) {
				Arrays.fill(rows, null);
				next_x = 0;

				for (var research : Registry.ANDROID_RESEARCH.getValues()) {
					if (research.getPrerequisites().size() == 0) {
						dive(cap, research, 0);
						float max = 0;

						for (float v : rows_width)
							max = Math.max(max, v);

						for (var button_list : created_buttons) {
							int count = 0;

							for (int i = 0; i < button_list.length; i++) {
								if (button_list[i] == null) {
									count = i;
									break;
								}
							}

							if (count > 0) {
								float this_x = next_x + max / 2f - (count * 22f) / 2f;

								for (int i = 0; i < count; i++) {
									button_list[i].setPos(this_x, 2);
									this_x += 22;
								}
							}
						}

						for (var v : created_buttons)
							Arrays.fill(v, null);

						next_x += max;
						Arrays.fill(rows_width, 0);
						Arrays.fill(created_buttons_idx, 0);
					}
				}

				seen.clear();
			}
		});

		canvas.setDock(Dock.FILL);

		var bottom = new EditablePanel(this, research, 0, 0, 0, 20);
		var close = new ButtonPanel(this, bottom, 0, 0, 90, 20, new TranslatableComponent("otm.container.matter_panel.close"));
		bottom.setDock(Dock.BOTTOM);
		close.setDock(Dock.RIGHT);
		close.bindOnPress(research::remove);
		bottom.setDockMargin(0, 0, 4, 4);

		canvas.setDockMargin(4, 4, 4, 4);

		research.toScreenCenter();
		addPanel(research);
	}

	@Override
	public void resize(Minecraft minecraft, int p_96576_, int p_96577_) {
		super.resize(minecraft, p_96576_, p_96577_);

		if (research != null) {
			var window = minecraft.getWindow();
			research.setSize(window.getGuiScaledWidth() * 0.8f, window.getGuiScaledHeight() * 0.8f);
			research.toScreenCenter();
		}
	}

	@Nullable
	@Override
	protected FramePanel makeMainFrame() {
		var frame = new FramePanel(this, null, 0, 0, FRAME_WIDTH, FRAME_HEIGHT, getTitle());

		var button = new ButtonPanel(this, frame, 0, 0, 90, 20, new TranslatableComponent("otm.gui.android_research"));
		button.setDock(Dock.BOTTOM);
		button.bindOnPress(this::openResearchTree);
		button.setDockMargin(10, 0, 10, 0);

		return frame;
	}
}
