package ru.dbotthepony.mc.otm.menu

import net.minecraft.world.SimpleContainer
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.item.ItemStack
import net.minecraftforge.common.ForgeHooks
import net.minecraftforge.energy.CapabilityEnergy
import ru.dbotthepony.mc.otm.Registry
import ru.dbotthepony.mc.otm.block.entity.BlockEntityChemicalGenerator
import ru.dbotthepony.mc.otm.menu.data.IntDataContainer
import ru.dbotthepony.mc.otm.menu.slot.MatterySlot
import ru.dbotthepony.mc.otm.menu.widget.LevelGaugeWidget
import ru.dbotthepony.mc.otm.menu.widget.ProgressGaugeWidget

class ChemicalGeneratorMenu @JvmOverloads constructor(id: Int, inv: Inventory, tile: BlockEntityChemicalGenerator? = null)
	: MatteryMenu(Registry.Menus.CHEMICAL_GENERATOR, id, inv, tile) {

	val container = tile?.container ?: SimpleContainer(2)

	val fuelSlot = object : MatterySlot(container, 0) {
		override fun mayPlace(p_40231_: ItemStack): Boolean {
			return ForgeHooks.getBurnTime(p_40231_, null) > 0
		}
	}

	val batterySlot = object : MatterySlot(container, 1) {
		override fun mayPlace(p_40231_: ItemStack): Boolean {
			return p_40231_.getCapability(CapabilityEnergy.ENERGY).isPresent
		}
	}

	val progress = ProgressGaugeWidget(this) { 1f - tile!!.working_ticks.toFloat() / tile.working_ticks_total }
	val energy = LevelGaugeWidget(this, tile?.energy)
	val burn_time = IntDataContainer()

	init {
		addSlot(fuelSlot)
		addSlot(batterySlot)
		addDataSlots(burn_time)
		addInventorySlots()
	}

	override fun broadcastChanges() {
		super.broadcastChanges()
		progress.updateServer()
		burn_time.value = (tile as BlockEntityChemicalGenerator).working_ticks
	}

	override fun getWorkingSlotStart(): Int {
		return 0
	}

	override fun getWorkingSlotEnd(): Int {
		return 2
	}
}
